% sound_mode.m
%
% A simple problem using BLT (and Yices) as a sound and complete decision
% procedure for bounded LIA.
%
% In this problem we prove that there is no lattice point in the
% given (very narrow) rectangle using BLT. We pass the -y parameter to BLT
% in order to use the Yices library LRA decision procedure to provide pruning
% decisions that are sound, i.e. free from floating point error.

mss_config('default_solver', 'blt');  % use BLT for linear integer arithmetic
mss_config('blt_params', '-y');       % enable Yices support

% ----------------------------------------------
% Setup a simple, 2d, unsatisfiable

l = [20.5; 20];                       % hyper-rectangle w/o Z-lattice points
u = [20.75; 21];
A = [0 1; 1 0];                       % the standard 2d Z-lattice

x = symbolic('x', 2, 1, 'integer');   % symbolic lattice vector coefficients

msg = 'Trying UNSAT problem..'
[r,c] = check_sat(l<=A*x & A*x<=u);   % Use BLT w/ Yices to decide if there is
                                      % a solution. This problem involves only
                                      % one search step that prunes the entire
                                      % search space, hence the problem is
                                      % UNSAT.

if r
    'SAT'
    c.x
else
    'UNSAT'
end

% ----------------------------------------------
% Modify the problem so that it is satisfiable

msg = 'Trying SAT problem..'

u = [30; 30];
[r,c] = check_sat(l<=A*x & A*x<=u);

if r
    'SAT'
    c.x
else
    'UNSAT'
end
