% Input: an array of convex equations, symbolic values for the matching "active"
% indicator for building learned clauses.
%
% Output: an unsatisfied core problem
%
% TODO: sort the input 'cc' constraints by their individual slacks from the
% optimial single-slack point.  This gives a measure of infeasiblity and
% arguably good ordering for unsat core construction better than the (arbitrary)
% input ordering.
function cert = unsatCore(cc,b)
    [nr nr] = size(cc);
    s       = symbolic('unsatCoreSlack','real');
    I0      = cc(1);
    cert    = [ b(1) ];
    Brest   = b(2:nr);
    Irest   = cc(2:nr);
    [nrIrest nrIrest] = size(Irest);

    [tsat]   = convex_opt(s, I0);
    for i = [1:nrIrest]
        if tsat
            I0 = [I0 Irest(i)];
            cert = or(cert,Brest(i));
            [nrI0 nrI0] = size(I0);
            iSlack = [];
            for k = [1:nrI0]
                iSlack = [iSlack (I0(k) + s) ];
            end
            [tsat] = convex_opt(s, [iSlack, (-s)]);
        end
    end
end

% For example purposes only, a non-minimal unsat core.
% Observe how the lack of information being fed back from the theory solver to
% the SAT solver results in poor performance.
% function cert = unsatCore(cc,b)
%     cert = false;
%     [nr nr] = size(b);
%     for i = [1:nr]
%         cert = or(cert,b(i));
%     end
% end
