% m_true_table(m,x) returns a predicate that is true if @x@
% constains exactly @m@ true bits.  This algorithm is designed
% to support cases where the values of @x@ are symbolic, and uses
% dynamic programming to minimize the number of sub-computations
% compared to @m_true@.  Despite this, it still is slower than
% @m_true@, as array assignments are currently very slow in @mss@.
function r = m_true_table(m, x)
  n = length(x);

  h = n-m;
  p = false_accum(x,h);
  p = next(h+1,x,p);
  for j = 2:m
    p = next(h+1,x(j:end),p);
  end

  r = p(h+1);
end

% Returns array p with n+1 elements such that p(i) holds
% if all entries 1:i-1 are false.
function p = false_accum(x,n)
  last = true;
  p = true(n+1, 1);
  for i = 1:n
    last = last & ~x(i);
    p(i+1) = last;
  end
end

function q = next(cnt,x,prev)
  % Get number of elements needed for top.
  q = false(cnt, 1);
  last = false;
  % We maintain the invariant, that p(i) is true if x(1:i)
  % contains j-1 true predicates.
  for i = 1:cnt
    r = x(i);
    last = (r & prev(i)) | (~r & last);
    q(i) = last;
  end
end