#include <stdio.h>
#include <sym-api.h>

const int sz = 11;

int find_majority( int* arr, int n );

int main()
{
  int* arr = (int*) mss_fresh_uint32_array( "x", sz );

  // Assume that 'maj' is the majority element
  int maj = mss_fresh_uint32( "maj" );
  int count = 0;
  for( int i=0; i<sz; i++ ) {
    if( maj == arr[i] ) { count++; }
  }
  mss_assume( (2*count) > sz );

  // Call the algorithm
  int c = find_majority( arr, sz );

  // Negate the goal for verification
  int p = c != maj;

  // Write the problem statement
  mss_write_yices( p, "majority.yices" );

  // Call Yices directly for the proof
  struct mss_model* mdl;
  mdl = mss_check_sat( p );
  if( mdl != NULL ) {
    printf("COUNTEREXAMPLE:\n");
    mss_print_model(mdl);

    uint32_t arr_sz;
    int* mdl_arr = (int*) mss_model_get_uint32_array( mdl, "x", &arr_sz );
    int mdl_c = find_majority( mdl_arr, arr_sz );

    printf( "Calculated majority 0x%x\n", mdl_c );
    free(mdl_arr);

  } else {
    printf( "Succesfully verified\n" );
  }

  free(mdl);
  free(arr);

  return c;
}
