#ifndef _SYM_API_H_
#define _SYM_API_H_

#if defined(c_plusplus) || defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>
#include <stdlib.h>

#define NO_INL __attribute__ ((noinline))

  uint8_t  grackle_fresh_uint8  ( const char* name ) NO_INL;
  uint16_t grackle_fresh_uint16 ( const char* name ) NO_INL;
  uint32_t grackle_fresh_uint32 ( const char* name ) NO_INL;
  uint64_t grackle_fresh_uint64 ( const char* name ) NO_INL;
  float    grackle_fresh_float  ( const char* name ) NO_INL;
  double   grackle_fresh_double ( const char* name ) NO_INL;

  uint8_t*  grackle_fresh_uint8_array ( const char* name, uint32_t num ) NO_INL;
  uint16_t* grackle_fresh_uint16_array( const char* name, uint32_t num ) NO_INL;
  uint32_t* grackle_fresh_uint32_array( const char* name, uint32_t num ) NO_INL;
  uint64_t* grackle_fresh_uint64_array( const char* name, uint32_t num ) NO_INL;
  float*    grackle_fresh_float_array ( const char* name, uint32_t num ) NO_INL;
  double*   grackle_fresh_double_array( const char* name, uint32_t num ) NO_INL;

  void grackle_print_symbolic_uint8 ( uint8_t sym ) NO_INL;
  void grackle_print_symbolic_uint16( uint16_t sym ) NO_INL;
  void grackle_print_symbolic_uint32( uint32_t sym ) NO_INL;
  void grackle_print_symbolic_uint64( uint64_t sym ) NO_INL;
  void grackle_print_symbolic_float ( float sym ) NO_INL;
  void grackle_print_symbolic_double( double sym ) NO_INL;
  void grackle_print_symbolic_pointer( void* sym ) NO_INL;

  void grackle_dump_mem( void ) NO_INL;

  struct grackle_model;

  /**
   * Check satisfiabiility of predicate, returning grackle_model if it is satisfiable,
   * and NULL if unsatisfiable.
   */
  struct grackle_model* grackle_check_sat( uint32_t pred ) NO_INL;
  struct grackle_model* grackle_check_sat_raw( uint32_t pred ) NO_INL;

  uint8_t  grackle_model_get_uint8 ( struct grackle_model* mdl, const char* name ) NO_INL;
  uint16_t grackle_model_get_uint16( struct grackle_model* mdl, const char* name ) NO_INL;
  uint32_t grackle_model_get_uint32( struct grackle_model* mdl, const char* name ) NO_INL;
  uint64_t grackle_model_get_uint64( struct grackle_model* mdl, const char* name ) NO_INL;
  float    grackle_model_get_float ( struct grackle_model* mdl, const char* name ) NO_INL;
  double   grackle_model_get_double( struct grackle_model* mdl, const char* name ) NO_INL;

  uint8_t*  grackle_model_get_uint8_array ( struct grackle_model* mdl, const char* name, uint32_t* num ) NO_INL;
  uint16_t* grackle_model_get_uint16_array( struct grackle_model* mdl, const char* name, uint32_t* num ) NO_INL;
  uint32_t* grackle_model_get_uint32_array( struct grackle_model* mdl, const char* name, uint32_t* num ) NO_INL;
  uint64_t* grackle_model_get_uint64_array( struct grackle_model* mdl, const char* name, uint32_t* num ) NO_INL;
  float*    grackle_model_get_float_array ( struct grackle_model* mdl, const char* name, uint32_t* num ) NO_INL;
  double*   grackle_model_get_double_array( struct grackle_model* mdl, const char* name, uint32_t* num ) NO_INL;

  void grackle_print_model( struct grackle_model* mdl ) NO_INL;
  void grackle_write_smtlib2( uint32_t pred, const char* filename ) NO_INL;
  void grackle_write_yices( uint32_t pred, const char* filename ) NO_INL;
  uint32_t grackle_prove_obligations(void) NO_INL;
  uint32_t* grackle_proof_obligations( uint32_t* num ) NO_INL;

  void grackle_assume( uint32_t pred );
  void call_grackle_assert( uint32_t pred, ... );

#define S_(x) #x
#define S(x) S_(x)
#define grackle_assert_aux( x, xstr, line, file ) \
  call_grackle_assert( x, file ":" line " '" xstr "'" )
#define grackle_assert(x) grackle_assert_aux( x, #x, S(__LINE__), __FILE__ )

#if defined(c_plusplus) || defined(__cplusplus)
}
#endif

#endif
